import gulp from 'gulp';
import plugins from 'gulp-load-plugins';
import browser from 'browser-sync';
import rimraf from 'rimraf';
import panini from 'panini';
import yargs from 'yargs';
import lazypipe from 'lazypipe';
import inky from 'inky';
import fs from 'fs';
import siphon from 'siphon-media-query';
import replace from 'gulp-replace';

const $ = plugins();

const ENV = yargs.argv.env || 'local'; // local | staging | production
const IS_ENV_LOCAL = ENV === 'local';

// Build the "dist" folder by running all of the below tasks
gulp.task('build', gulp.series(clean, pages, sass, inline));

// Build emails, run the server, and watch for file changes
gulp.task('default', gulp.series('build', server, watch));

// Delete the "dist" folder
// This happens every time a build starts
function clean(done) {
  rimraf('dist', done);
}

// Compile layouts, pages, and partials into flat HTML files
// Then parse using Inky templates
function pages() {
  return gulp
    .src(['src/pages/**/*.html', '!src/pages/archive/**/*.html'])
    .pipe(
      $.if(
        ENV !== 'local',
        replace(/{{([^>$].*?)}}/g, '__START_HANDLEBAR__$1__END_HANDLEBAR__'),
      ),
    )
    .pipe(
      panini({
        root: 'src/pages',
        layouts: 'src/layouts',
        partials: 'src/partials',
        helpers: 'src/helpers',
        data: 'src/data',
      }),
    )
    .pipe(
      $.if(
        ENV !== 'local',
        replace(/__START_HANDLEBAR__(.*?)__END_HANDLEBAR__/g, '{{$1}}'),
      ),
    )
    .pipe(inky())
    .pipe(gulp.dest('dist'));
}

// Reset Panini's cache of layouts and partials
function resetPages(done) {
  panini.refresh();
  done();
}

// Compile Sass into CSS
function sass() {
  return gulp
    .src('src/assets/scss/**/*.scss')
    .pipe($.if(IS_ENV_LOCAL, $.sourcemaps.init()))
    .pipe(
      $.sass({
        includePaths: ['node_modules/foundation-emails/scss'],
      }).on('error', $.sass.logError),
    )
    .pipe(
      $.if(
        !IS_ENV_LOCAL,
        $.uncss({
          html: ['dist/**/*.html'],
        }),
      ),
    )
    .pipe($.if(IS_ENV_LOCAL, $.sourcemaps.write()))
    .pipe(gulp.dest('dist/css'));
}

// Inline CSS and minify HTML
function inline() {
  return gulp
    .src('dist/**/*.html')
    .pipe($.if(!IS_ENV_LOCAL, inliner()))
    .pipe(gulp.dest('dist'));
}

// Start a server with LiveReload to preview the site in
function server(done) {
  browser.init({
    server: {
      baseDir: 'dist',
      directory: true,
    },
  });
  done();
}

// Watch for file changes
function watch() {
  gulp
    .watch('src/pages/**/*.html')
    .on('all', gulp.series(pages, inline, browser.reload));
  gulp
    .watch(['src/layouts/**/*', 'src/data/**/*', 'src/partials/**/*'])
    .on('all', gulp.series(resetPages, pages, inline, browser.reload));
  gulp
    .watch(['../scss/**/*.scss', 'src/assets/scss/**/*.scss'])
    .on('all', gulp.series(resetPages, sass, pages, inline, browser.reload));
}

function extractMediaQuery(file) {
  const css = fs.readFileSync(file).toString();
  return siphon(css);
}
// Inlines CSS into HTML, adds media query CSS into the <style> tag of the email, and compresses the HTML
function inliner() {
  var pipe = lazypipe()
    .pipe($.inlineCss, {
      applyStyleTags: false,
      removeStyleTags: false,
      preserveMediaQueries: true,
      removeLinkTags: false,
    })
    .pipe(
      $.replace,
      '<!-- <style> -->',
      `<style>${extractMediaQuery('dist/css/app.css')}</style>`,
    )
    .pipe(
      $.replace,
      '<!-- <style-shipper> -->',
      `<style>${extractMediaQuery('dist/css/shipper.css')}</style>`,
    )
    .pipe(
      $.replace,
      '<link rel="stylesheet" type="text/css" href="css/app.css">',
      '',
    )
    .pipe(
      $.replace,
      '<link rel="stylesheet" type="text/css" href="css/shipper.css">',
      '',
    )
    .pipe($.htmlmin, {
      collapseWhitespace: true,
      minifyCSS: true,
    });

  return pipe();
}
