'use strict';

const fs = require('fs');
const path = require('path');

const SendGridAPI = require('./SendGridAPI');
const productionConfig = require('./config.prod.json');
const stagingConfig = require('./config.staging.json');

const { prepareTestData } = require('./utils');

const BUILD_DIR = path.join(__dirname, '..', 'dist');

function getTitle(html) {
  const result = /<title>([^<]+)<\/title>/.exec(html);
  return result ? result[1] : '';
}

function getTemplateContent(project, template) {
  const templatePath = path.join(BUILD_DIR, project, `${template}.html`);

  if (!fs.existsSync(templatePath)) {
    throw new Error(`Cannot find html file for "${templatePath}".`);
  }

  return fs.readFileSync(templatePath, { encoding: 'utf8' });
}

async function publish({ project, env, version }) {
  const config = env === 'production' ? productionConfig : stagingConfig;

  if (!config[project]) {
    throw Error(`Cannot find project under name "${project}".`);
  }

  try {
    const api = new SendGridAPI(project);

    for (const template in config[project]) {
      const templateId = config[project][template];
      const html = getTemplateContent(project, template);

      const { versions = [] } = await api.getTemplate(templateId);
      const prevVersion = versions.find((x) => x.name === version);

      if (prevVersion) {
        console.log(`Updating template version "${version}" in "${template}".`);

        await api.updateTemplateVersion({
          html,
          templateId,
          name: version,
          versionId: prevVersion.id,
          subject: getTitle(html),
          testData: prepareTestData(),
        });
      } else {
        console.log(`Creating template version "${version}" in "${template}".`);

        await api.createTemplateVersion({
          html,
          templateId,
          name: version,
          isActive: true,
          subject: getTitle(html),
          testData: prepareTestData(),
        });
      }
    }
  } catch (e) {
    console.error(e.message);
    process.exit(1);
  }
}

module.exports = { publish };
